@@ -1,15 +1,14 @@
-/*
- * Copyright (C) 2007 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010,2011 NORDUnet A/S
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2010-2011, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #ifndef SYS_SOLARIS9
 #include <stdint.h>
 #endif
+#ifdef __linux__
+#include <sys/syscall.h>
+#include <unistd.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -20,6 +19,7 @@
 #include <syslog.h>
 #include <errno.h>
 #include <assert.h>
+#include <pthread.h>
 #include "debug.h"
 #include "util.h"
 
@@ -28,10 +28,9 @@ static uint8_t debug_level = DBG_INFO;
 static char *debug_filepath = NULL;
 static FILE *debug_file = NULL;
 static int debug_syslogfacility = 0;
-#if defined(WANT_FTICKS)
 static int fticks_syslogfacility = 0;
-#endif
 static uint8_t debug_timestamp = 0;
+static uint8_t debug_tid = 0;
 
 void debug_init(char *ident) {
     debug_file = stderr;
@@ -63,6 +62,10 @@ void debug_timestamp_on() {
     debug_timestamp = 1;
 }
 
+void debug_tid_on() {
+    debug_tid = 1;
+}
+
 uint8_t debug_get_level() {
     return debug_level;
 }
@@ -109,17 +112,13 @@ int debug_set_destination(char *dest, int log_type) {
 		debugx(1, DBG_ERR, "Unknown syslog facility %s", dest);
 	    if (log_type != LOG_TYPE_FTICKS)
 		debug_syslogfacility = facvals[i];
-#if defined(WANT_FTICKS)
             else if (log_type == LOG_TYPE_FTICKS)
 		fticks_syslogfacility = facvals[i];
-#endif
 	} else {
             if (log_type != LOG_TYPE_FTICKS)
                 debug_syslogfacility = LOG_DAEMON;
-#if defined(WANT_FTICKS)
             else if (log_type == LOG_TYPE_FTICKS)
                 fticks_syslogfacility = 0;
-#endif
     	}
 	openlog(debug_ident, LOG_PID, debug_syslogfacility);
 	return 1;
@@ -153,9 +152,31 @@ void debug_reopen_log() {
 
 void debug_logit(uint8_t level, const char *format, va_list ap) {
     struct timeval now;
-    char *timebuf;
+    char *timebuf, *tidbuf, *tmp = NULL;
     int priority;
 
+    if (debug_tid) {
+#ifdef __linux__
+        pid_t tid = syscall(SYS_gettid);
+        tidbuf = malloc(3*sizeof(tid)+1);
+        sprintf(tidbuf, "%u", tid);
+#else
+        pthread_t tid = pthread_self();
+        uint8_t *ptid = (uint8_t *)&tid;
+        int i;
+
+        tidbuf = malloc((2*sizeof(tid)+1));
+        tmp = tidbuf;
+        for (i = sizeof(tid)-1; i >= 0; i--) {
+            tmp += sprintf(tmp, "%02x", ptid[i]);
+        }
+#endif
+        tmp = malloc(strlen(tidbuf) + strlen(format) + 4);
+        sprintf(tmp, "(%s) %s", tidbuf, format);
+        format = tmp;
+        free(tidbuf);
+    }
+
     if (debug_syslogfacility) {
 	switch (level) {
 	case DBG_DBG:
@@ -188,6 +209,7 @@ void debug_logit(uint8_t level, const char *format, va_list ap) {
 	vfprintf(debug_file, format, ap);
 	fprintf(debug_file, "\n");
     }
+    free(tmp);
 }
 
 void debug(uint8_t level, char *format, ...) {
@@ -236,7 +258,6 @@ void debugerrnox(int err, uint8_t level, char *format, ...) {
     exit(err);
 }
 
-#if defined(WANT_FTICKS)
 void fticks_debug(const char *format, ...) {
     int priority;
     va_list ap;
@@ -246,10 +267,9 @@ void fticks_debug(const char *format, ...) {
     else {
     	priority = LOG_DEBUG | fticks_syslogfacility;
     	vsyslog(priority, format, ap);
-    	va_end(ap);
     }
+    va_end(ap);
 }
-#endif
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
 /* End: */
