@@ -1,3 +1,6 @@
+/* Copyright (c) 2007-2008, UNINETT AS */
+/* See LICENSE for licensing information. */
+
 #define CONF_STR 1
 #define CONF_CBK 2
 #define CONF_MSTR 3
@@ -23,3 +26,8 @@ int popgconf(struct gconffile **cf);
 void freegconfmstr(char **mstr);
 void freegconf(struct gconffile **cf);
 struct gconffile *openconfigfile(const char *file);
+int unhex(char *s, uint8_t process_null);
+
+/* Local Variables: */
+/* c-file-style: "stroustrup" */
+/* End: */
