@@ -6,10 +6,11 @@
 #include <stdint.h>
 #include <pthread.h>
 #include <regex.h>
+#include <netinet/in.h>
 #include "list.h"
-#include "tlv11.h"
 #include "radmsg.h"
 #include "gconfig.h"
+#include "rewrite.h"
 
 #define DEBUG_LEVEL 2
 
@@ -17,6 +18,7 @@
 
 /* MAX_REQUESTS must be 256 due to Radius' 8 bit ID field */
 #define MAX_REQUESTS 256
+#define MAX_LOSTRQS 16
 #define REQUEST_RETRY_INTERVAL 5
 #define REQUEST_RETRY_COUNT 2
 #define DUPLICATE_INTERVAL REQUEST_RETRY_INTERVAL * REQUEST_RETRY_COUNT
@@ -72,6 +74,13 @@ enum rsp_server_state {
     RSP_SERVER_STATE_FAILING
 };
 
+enum rsp_statsrv {
+	RSP_STATSRV_OFF = 0,
+	RSP_STATSRV_ON,
+	RSP_STATSRV_MINIMAL,
+	RSP_STATSRV_AUTO
+};
+
 struct options {
     char *pidfile;
     char *logdestination;
@@ -135,17 +144,22 @@ struct clsrvconf {
     int hostaf;
     char *portsrc;
     struct list *hostports;
-    char *secret;
+    char *confsecret;
+    uint8_t *secret;
+    int secret_len;
     char *tls;
     char *matchcertattr;
     regex_t *certcnregex;
     regex_t *certuriregex;
+    regex_t *certdnsregex;
+    struct in6_addr certipmatch;
+    int certipmatchaf;
     char *confrewritein;
     char *confrewriteout;
     char *confrewriteusername;
     struct modattr *rewriteusername;
     char *dynamiclookupcommand;
-    uint8_t statusserver;
+    enum rsp_statsrv statusserver;
     uint8_t retryinterval;
     uint8_t retrycount;
     uint8_t dupinterval;
@@ -183,7 +197,7 @@ struct server {
     pthread_mutex_t lock;
     pthread_t clientth;
     uint8_t clientrdgone;
-    struct timeval lastconnecttry;
+    struct timeval connecttime;
     struct timeval lastreply;
     enum rsp_server_state state;
     uint8_t lostrqs;
@@ -211,19 +225,6 @@ struct realm {
     struct list *accsrvconfs;
 };
 
-struct modattr {
-    uint8_t t;
-    char *replacement;
-    regex_t *regex;
-};
-
-struct rewrite {
-    uint8_t *removeattrs;
-    uint32_t *removevendorattrs;
-    struct list *addattrs;
-    struct list *modattrs;
-};
-
 struct protodefs {
     char *name;
     char *secretdefault;
@@ -237,7 +238,7 @@ struct protodefs {
     void (*setprotoopts)(struct commonprotoopts *);
     char **(*getlistenerargs)();
     void *(*listener)(void*);
-    int (*connecter)(struct server *, struct timeval *, int, char *);
+    int (*connecter)(struct server *, int, char *);
     void *(*clientconnreader)(void*);
     int (*clientradput)(struct server *, unsigned char *);
     void (*addclient)(struct client *);
@@ -248,11 +249,6 @@ struct protodefs {
 
 #define RADLEN(x) ntohs(((uint16_t *)(x))[1])
 
-#define ATTRTYPE(x) ((x)[0])
-#define ATTRLEN(x) ((x)[1])
-#define ATTRVAL(x) ((x) + 2)
-#define ATTRVALLEN(x) ((x)[1] - 2)
-
 struct clsrvconf *find_clconf(uint8_t type, struct sockaddr *addr, struct list_node **cur);
 struct clsrvconf *find_srvconf(uint8_t type, struct sockaddr *addr, struct list_node **cur);
 struct clsrvconf *find_clconf_type(uint8_t type, struct list_node **cur);
@@ -265,7 +261,7 @@ void freerq(struct request *rq);
 int radsrv(struct request *rq);
 void replyh(struct server *server, unsigned char *buf);
 struct addrinfo *resolve_hostport_addrinfo(uint8_t type, char *hostport);
-uint8_t *radattr2ascii(struct tlv *attr);
+uint8_t *radattr2ascii(struct tlv *attr); /* TODO: mv this to radmsg? */
 pthread_attr_t pthread_attr;
 
 /* Local Variables: */
