@@ -1,10 +1,6 @@
-/*
- * Copyright (C) 2008 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2008, UNINETT AS
+ * Copyright (c) 2010, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 struct tlv {
     uint8_t t;
@@ -20,7 +16,7 @@ struct list *copytlvlist(struct list *);
 void freetlvlist(struct list *);
 void rmtlv(struct list *, uint8_t);
 uint8_t *tlv2str(struct tlv *tlv);
-uint8_t *tlv2buf(uint8_t *, const struct tlv *tlv);
+struct tlv *resizetlv(struct tlv *, uint8_t);
 
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
